(function ($) {
  // image processing + reaction time is ~ 350ms
  // typing typically needs ~480ms or more (for average typers)
  const DOHERTY_THRESHOLD = 400, DEBOUNCE_TIME = DOHERTY_THRESHOLD - 50;

  let loaded = false;

  function debounce (func, milliseconds = DEBOUNCE_TIME) {
    let timeout = null;

    return function (...params) {
      if (milliseconds) {
        if (timeout !== null) {
          clearTimeout(timeout);
        }
  
        timeout = setTimeout(func, milliseconds, ...params);
      } else {
        func(...params);
      }
    };
  }

  function hashString(str) {
    var hash = 0n;

    if (str.length > 0) {
      for (let i = 0; i < str.length; i++) {
        hash = (hash * 31n + BigInt(str.charCodeAt(i)));
      }
    }

    return (hash % 0xFFFFFFFFn).toString(16);
  }

  window.addEventListener('load', function () {
    loaded = true;
  });

  function checkFormChange (form) {
    if (!loaded) {
      return;
    }

    forEachGlobalElement((element, inclusions) => {
      let newHash = getFormDataHash(form, inclusions);

      if (!element.dataset.formDataHash) {
        element.dataset.formDataHash = newHash;
      } else if (element.dataset.formDataHash !== newHash) {
        element.dataset.formDataHash = newHash;

        checkFormChange.debounced[element.id] = checkFormChange.debounced[element.id] || debounce(function () {
          element.dispatchEvent(new Event('change'));
        });

        checkFormChange.debounced[element.id]();
      }
    });
  }

  checkFormChange.debounced = {};

  function forEachGlobalElement(func) {
    if (drupalSettings && drupalSettings.monarch_data_entity && drupalSettings.monarch_data_entity.global_change) {
      Object.keys(drupalSettings.monarch_data_entity.global_change).forEach(id => {
        let element = document.getElementById(id);

        if (element) {
          func(element, drupalSettings.monarch_data_entity.global_change[id]);
        }
      });
    }
  }

  function getFormDataHash(form, inclusions = null) {
    let formValues = Array.from(new FormData(form));

    if (inclusions) {
      formValues = formValues.filter(v => {
        let parts = v[0].split(/[^a-z0-9_]+/gi).filter(Boolean);

        return inclusions[parts[0]];
      });
    }

    return hashString(JSON.stringify(formValues));
  }

  Drupal.behaviors.viewWidgetNotifier = Drupal.behaviors.viewWidgetNotifier || {
    attach: function attach (context) {
      $('form').each(function () {
        checkFormChange(this);
      });

      once('view-input-view-embed-form-changes', 'form', context).forEach(function (form) {
        let interval = setInterval(() => {
          if (!form) {
            clearInterval(interval);
          } else {
            checkFormChange(form);
          }
        }, 80);
      });
    },
  };
})(jQuery);
